#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const makeDir = require("make-dir");
const meow = require("meow");
const tempy = require("tempy");
const execa = require("execa");
const util_1 = require("./util");
const createSdk = ({ outDir, service }) => __awaiter(void 0, void 0, void 0, function* () {
    // Get a temporary folder name
    const tempFolder = tempy.directory();
    const servicePath = [process.cwd(), service, 'src'].filter(Boolean);
    const serviceLocation = path.join(...servicePath);
    // Create the apidocs for the service and write them to the temporary folder
    yield execa('apidoc', ['-i', serviceLocation, '-o', tempFolder], {
        stdio: 'inherit',
        preferLocal: true,
        localDir: __dirname,
        reject: false
    });
    // Retrieve the focs from the temporary folder
    const docs = util_1.getDocs(tempFolder);
    const serviceName = yield util_1.getServiceName();
    const baseProjectPath = path.join(__dirname, 'base');
    const sdkPath = path.join(process.cwd(), outDir);
    const typingPath = path.join(sdkPath, 'index.d.ts');
    // Create sdk folder
    yield makeDir(sdkPath);
    // Create typing file
    fs.writeFileSync(typingPath, '');
    const apiParams = [];
    for (const doc of docs) {
        const newInterface = util_1.createInterface(doc);
        fs.appendFileSync(typingPath, newInterface);
        apiParams.push(util_1.mapApiDocs(doc, serviceName));
    }
    const index = fs.readFileSync(path.join(baseProjectPath, 'index.js')).toString();
    const packageJson = fs.readFileSync(path.join(baseProjectPath, 'package.json').toString());
    fs.writeFileSync(path.join(sdkPath, 'index.js'), index.replace('$api', JSON.stringify(apiParams, null, '\t')));
    fs.writeFileSync(path.join(sdkPath, 'package.json'), packageJson);
});
const cli = meow(`
	Usage
		$ lambda-sdk-generator

	Options
		--outDir, -o path where to put sdk
		--service, -s location of the service to generate docs for

	Examples
		$ lambda-sdk-generator -s='services/store' -o='sdk/store'
`, {
    flags: {
        outDir: {
            type: 'string',
            alias: 'o',
            default: 'sdk'
        },
        service: {
            type: 'string',
            alias: 's'
        }
    }
});
createSdk(cli.flags)
    .then(() => console.log(`SDK has been created in the \`${cli.flags.outDir}\` folder`))
    .catch(err => console.log('Something went wrong while generating the SDK \n', err));
